/* Copyright 2002  Thomas Pfenning                                                      */
/* I license this source code for commercial or non-commercial use free of              */
/* any charge. You are not allowed, however, to redistribute this source code or parts  */
/* of this source code under any other license. If you wish to make the source code to  */
/* a derived product available you must break out all pieces derived from this code and */
/* make them available separatly with this Copyright notice intact.                     */
/* The code serves no particular purpose, which means it can not be wrong or violate    */
/* any existing rights.                                                                 */

#include "pcmconverter.h"

/* Global definitions and buffers */

extern void PrintWaveHeader (WAV *, RIFFDATA *);
extern int ReadBuffer12 (unsigned char *, int);

#define BLOCKS 4096*4
#define SIZE  12 * BLOCKS     /* Make this a multiple of a generic disk blocksize */
unsigned char buf[SIZE], obuf[SIZE];
char unsigned *r,*s;
int inp = 0;        
int outp = 1;      
unsigned long totalread;

int main (int argc, char *argv[]) {
int i,j;
struct stat filestatus;
unsigned long pointer, percent, scale;
unsigned long setsize;
int ex = 0;      /* default is WAVE_FORMAT_PCM for the time being */
unsigned int hz = 96000;  /* default is 96000 Hz sampling rate */
int bits =16;


  if (argc < 3) { 
    fprintf (stderr, "Argument(s) missing\n\n");
    fprintf (stderr, "%s\n", helptext);
    exit (-1);
  };

  /* parse all options, input file, and output file */
  i = 1;
  while ((argv[i][0] =='-') && (i < argc)) {
    if (strcmp (argv[i], "-ex") == 0) {
      ex = 1;         /* Deal with WAVEFORMATEXTENSIBLE */
      i++;
      goto loop;
    };
    if (strcmp (argv[i], "-48") == 0) {
      hz = 48000;     /* 48000 Hz sample rate */
      i++;
      goto loop;
    };
    fprintf (stderr, "Invalid argument\n\n");
    fprintf (stderr, "%s\n", helptext);
    exit (-1);
loop:
  };


  if ((argc - i) < 2) {
    fprintf (stderr, "Argument missing\n\n");
    fprintf (stderr, "%s\n", helptext);
    exit (-1);
  };

  /* assume next 2 arguments are input file and output file */
  if (strcmp (argv[i], argv[i+1]) == 0) {
    fprintf (stderr, "InputFile and Outputfile must be different\n\n");
    fprintf (stderr, "%s\n", helptext);
    exit (-1);
  };

  if ((inp = open (argv[i], O_RDONLY|O_BINARY)) == -1) { 
    fprintf (stderr, "Can't find input file %s\n", argv[i]);
    fprintf (stderr, "%s\n", helptext);
    exit (-1);
  };

  if ((outp = open (argv[i+1], O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, S_IRUSR|S_IWUSR)) == -1) { 
    fprintf (stderr, "Can't create output file %s\n", argv[i+1]);
    fprintf (stderr, "%s\n", helptext);
    exit (-1);
  }; 
  /* done parsing command line */


  if (fstat (inp, &filestatus) == -1) {
    fprintf (stderr, "Can't stat input file %s\n", argv[1]);
    exit (-1);
  };
  if (((filestatus.st_size / 12) * 12) != filestatus.st_size) { /* truncate to closest */
    filestatus.st_size = (filestatus.st_size /12) * 12;
    fprintf (stderr, "input file size is not a multiple of 12. Truncated to closest.\n");
  };

  if (read (inp, &wave, sizeof(wave)) == 44) {
    if ( wave.chunkid[0] == 'R' && wave.chunkid[1] == 'I' &&
         wave.chunkid[2] == 'F' && wave.chunkid[3] == 'F') {
      fprintf (stderr, "Input file has WAV header\n\n");
      fprintf (stderr, "%s\n", helptext);
      exit (-1);
    }
  }

  lseek (inp, 0, SEEK_SET);   /* file seems okay reset to beginning  */


  wave.chunkid[0] = 'R';
  wave.chunkid[1] = 'I';
  wave.chunkid[2] = 'F';
  wave.chunkid[3] = 'F';

  wave.chunksize = (filestatus.st_size * 12 ) / 12 + 36;

  wave.format[0] = 'W';
  wave.format[1] = 'A';
  wave.format[2] = 'V';
  wave.format[3] = 'E';

  wave.Id1[0] = 'f';
  wave.Id1[1] = 'm';
  wave.Id1[2] = 't';
  wave.Id1[3] = ' ';

  wave.Size1= 16;		/* PCM */
  wave.AudioFormat = 1;		/* PCM */
  wave.NumChannels = 2;
  wave.SampleRate = hz;
  wave.BitsPerSample = 24;
  wave.ByteRate = wave.NumChannels * wave.SampleRate * wave.BitsPerSample / 8;
  wave.BlockAlign = wave.NumChannels * wave.BitsPerSample / 8;

  /* Fill in the data chunk part of the the header */  
  rdata.Id2[0] = 'd';
  rdata.Id2[1] = 'a';
  rdata.Id2[2] = 't';
  rdata.Id2[3] = 'a';

  rdata.Size2 = (filestatus.st_size * 12) / 12;

  /* support for the new WAVE_FORMAT_EXTENSIBLE format */
  if (ex) {
    wavex.ExtraSize = 22;
    wavex.Format = wave;
    wavex.Format.Size1 = 40;
    wavex.Format.chunksize += 24;
    wavex.Format.AudioFormat = WAVE_FORMAT_EXTENSIBLE;
    wavex.Samples.wValidBitsPerSample = 24;
    wavex.dwChannelMask = SPEAKER_FRONT_RIGHT | SPEAKER_FRONT_LEFT;
    wavex.SubFormat = KSDATAFORMAT_SUBTYPE_PCM;

    if (write (outp, &wavex, sizeof(wavex)) < sizeof(wavex)) {
       fprintf (stderr, "Failed to write wav header\n");
       exit(-1);
    };

  } else {
    if (write (outp, &wave, sizeof(wave)) < sizeof(wave)) {
       fprintf (stderr, "Failed to write wav header\n");
       exit(-1);
    };
  };

  if (write (outp, &rdata, sizeof(rdata)) < sizeof(rdata)) {
     fprintf (stderr, "Failed to write wav header\n");
     exit(-1);
  };

  pointer = 0;
  percent = 0;
  setsize = 0;
  scale = rdata.Size2 / 1024;
  totalread = 0;
  while ((setsize = ReadBuffer12 (buf, SIZE)) > 0) {
    r = buf;
    s = obuf;
    totalread += setsize;
    for (i = 0; i < setsize / 12; i++) {     /* ReadBuffer12 returns multiples of 12 */
      s[0] = r[8];   /* reshuffle bytes into temporary location */
      s[1] = r[1];
      s[2] = r[0];
      s[3] = r[9];
      s[4] = r[3];
      s[5] = r[2];
      s[6] = r[10];
      s[7] = r[5];
      s[8] = r[4];
      s[9] = r[11];
      s[10] = r[7];
      s[11] = r[6];
      r += 12;
      s += 12;
    };
     write (outp, obuf, setsize);

     pointer += setsize / 1024;
     if (((pointer * 100) / scale) > percent) {
        percent = (pointer * 100) / scale;
        fprintf (stderr, "\r  %d %%", percent);
     };
  };
  if (totalread != filestatus.st_size) 
    fprintf (stderr, "WAV file corrupt , please rerun lpcm24\n");

  close(inp);
  close(outp);
  fprintf (stderr, "\n");
  exit(0);
};


void PrintWaveHeader (WAV *wave, RIFFDATA *rdata) {
  printf ("%c%c%c%c\n", wave->chunkid[0], wave->chunkid[1], wave->chunkid[2], wave->chunkid[3]);
  printf ("%d : Laenge\n", wave->chunksize);
  printf ("%c%c%c%c\n", wave->format[0], wave->format[1], wave->format[2], wave->format[3]);
  printf ("%c%c%c%c\n", wave->Id1[0], wave->Id1[1], wave->Id1[2], wave->Id1[3]);
  printf ("%d : PCM = 16\n", wave->Size1);
  printf ("%d : PCM = 1\n", wave->AudioFormat);
  printf ("%d : Number of channels\n", wave->NumChannels);
  printf ("%d : Sample Rate\n", wave->SampleRate);
  printf ("%d : Byte Rate\n", wave->ByteRate);
  printf ("%d : Blockalign\n", wave->BlockAlign);
  printf ("%d : Bits per Sample\n", wave->BitsPerSample);
  printf ("%c%c%c%c : \"data\"\n", rdata->Id2[0], rdata->Id2[1], rdata->Id2[2], rdata->Id2[3]);
  printf ("%d : Data Size\n", rdata->Size2);
};

int ReadBuffer12 (unsigned char *buf, int size) {
  int setsize;
  setsize = read (inp ,buf, size);
  if ( setsize < 0)
     return (-1);
  if (setsize < size) {                  /* result smaller than buffer */
        return ((setsize / 12) * 12);
  }; 
  return (size);
};
