#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#define WAVE_FORMAT_EXTENSIBLE 0xFFFE
#define WAVE_FORMAT_PCM 0x0001

#define SPEAKER_FRONT_LEFT             0x1
#define SPEAKER_FRONT_RIGHT            0x2
#define SPEAKER_FRONT_CENTER           0x4
#define SPEAKER_LOW_FREQUENCY          0x8
#define SPEAKER_BACK_LEFT              0x10
#define SPEAKER_BACK_RIGHT             0x20
#define SPEAKER_FRONT_LEFT_OF_CENTER   0x40
#define SPEAKER_FRONT_RIGHT_OF_CENTER  0x80
#define SPEAKER_BACK_CENTER            0x100
#define SPEAKER_SIDE_LEFT              0x200
#define SPEAKER_SIDE_RIGHT             0x400
#define SPEAKER_TOP_CENTER             0x800
#define SPEAKER_TOP_FRONT_LEFT         0x1000
#define SPEAKER_TOP_FRONT_CENTER       0x2000
#define SPEAKER_TOP_FRONT_RIGHT        0x4000
#define SPEAKER_TOP_BACK_LEFT          0x8000
#define SPEAKER_TOP_BACK_CENTER        0x10000
#define SPEAKER_TOP_BACK_RIGHT         0x20000
#define SPEAKER_RESERVED               0x80000000


typedef struct _WAV {
  char chunkid[4];		/* Contains the letters "RIFF"  */
  unsigned int chunksize;	/* total file length in bytes-8;raw data + 36 */
  char format[4];		/* Contains the letters "WAVE" */
  char Id1[4];			/* Contains the letters "fmt "  */
  unsigned int Size1;		/* PCM = 16 or 22 */
  unsigned short AudioFormat;   /* PCM == 1  or 65534 */
  unsigned short NumChannels;	/* Mono = 1 ; Stereo = 2 */
  unsigned int SampleRate;	/* 8000, 16000. 44100 etc. */
  unsigned int ByteRate;	/* SampleRate * NumChannels * BitsPerSample/8 */
  unsigned short BlockAlign;	/* NumChannels * BitsPerSample/8   */
  unsigned short BitsPerSample;	/* 8 bits = 8, 16 bits = 16, etc. */
} WAV;

typedef struct _RIFFDATA {
  char Id2[4];			/* Contains the letters "data"  */
  unsigned int Size2;		/* NumSamples * NumChannels * BitsPerSample/8 */
} RIFFDATA;

WAV wave;
RIFFDATA rdata;


typedef struct _GUID
{
    unsigned long Data1;
    unsigned short Data2;
    unsigned short Data3;
    unsigned char Data4[8];
} GUID;

typedef struct {
  WAV Format;
  unsigned short ExtraSize;
  union {
    unsigned short  wValidBitsPerSample;
    unsigned short  wSamplesPerBlock;
    unsigned short  wReserved;
  } Samples;
  unsigned long dwChannelMask; 
  GUID    SubFormat;
} WAVEX;

WAVEX wavex;



static GUID KSDATAFORMAT_SUBTYPE_PCM = {
            0x00000001,
            0x0000,
            0x0010,
            { 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71 }
        };


char helptext[] =
" usage: lpcm24 [-ex] [-48] InputFile Outputfile\n\n\n creates or overwrites an OutputFile in 24 bit WAV format\n from an InputFile with raw 24 bit LPCM data.\n\n If -ex is specified the Windows 2000 style WAV header is generated.\n If -48 is specified the header sample rate will be set to 48000 Hz. (Default is 96000 Hz)\n";

